@@ -9,6 +9,9 @@ from typing import List, Tuple
 from http.cookies import SimpleCookie
 import requests
 
+import re
+from typing import Optional, List
+
 try:
     from ptcookiechecker.modules.cookie_rules import COMMON_COOKIES
 except ImportError:
@@ -16,13 +19,14 @@ except ImportError:
 
 
 class CookieTester:
-
     def __init__(self):
         self.COMMON_COOKIES = COMMON_COOKIES
 
-    def run(self, response, args, ptjsonlib: object, test_cookie_issues: bool = True, filter_cookie: str = None):
+    def run(self, response, args, ptjsonlib: object, test_cookie_issues: bool = True, filter_cookie: str = None, tests=None):
         self.args = args
-        self.args.tests = getattr(args, "tests", [])
+        if tests:
+            self.args.tests = tests
+        self.args.tests = getattr(args, "tests", tests)
         self.args.redirects = getattr(args, "redirects", False)
         self.ptjsonlib = ptjsonlib
         self.use_json = self.args.json
@@ -58,8 +62,6 @@ class CookieTester:
             ptprinthelper.ptprint(f"Site returned no cookies", bullet_type="", condition=not self.use_json, indent=4)
             return
 
-
-
         for index, cookie in enumerate(cookie_list):
 
             if self.filter_cookie and (self.filter_cookie.lower() != cookie.name.lower()):
@@ -85,7 +87,7 @@ class CookieTester:
 
             if "IDENT" in self.args.tests:
                 node = self.ptjsonlib.create_node_object(
-                    "cookie",
+                    "web_cookie",
                     properties={
                         "name": cookie.name,
                         "is_web_cookie_secure_flag": cookie_secure_flag,
@@ -134,6 +136,9 @@ class CookieTester:
                         ptprinthelper.ptprint(f"Application sets any value passed in GET parameter into the cookie ", bullet_type="WARNING", condition=not self.use_json, indent=(self.base_indent+4))
                         self.ptjsonlib.add_vulnerability(vuln_code, node_key=self.node_key)
 
+                if "FPD" in self.args.tests:
+                    self._test_fpd_via_cookie_injection(self.args.url, cookie_name=cookie.name)
+
             ptprinthelper.ptprint(f"Domain: {cookie_domain}", bullet_type="TEXT", condition=not self.use_json, indent=self.base_indent)
 
             if self.test_cookie_issues:
@@ -160,10 +165,6 @@ class CookieTester:
                 ptprinthelper.ptprint(f"    Secure: {cookie_secure_flag}", bullet_type="TEXT", condition=not self.use_json, indent=self.base_indent)
                 ptprinthelper.ptprint(f"    HttpOnly: {cookie_http_flag}", bullet_type="TEXT", condition=not self.use_json, indent=self.base_indent)
 
-            if "FPD" in self.args.tests:
-                self._test__fpd_via_cookie_injection(self.args.url)
-
-
     def detect_duplicate_attributes(self, cookie_string):
         attributes = [attr.strip() for attr in cookie_string.split(';')]
         attribute_counts = {}
@@ -406,27 +407,12 @@ class CookieTester:
         cookie_expires = _expires.groups()[0] if _expires else ""
         return cookie_expires
 
-    def _test__fpd_via_cookie_injection(self, url: str) -> None:
+    def _test_fpd_via_cookie_injection(self, url: str, cookie_name=None) -> None:
         """
-        Test for potential Full Path Disclosure (FPD) by sending
-        malformed cookies and checking the server's response.
-
-        - Sends a cookie with an empty value.
-        - Sends a cookie with invalid characters (semicolon inside value).
-        - Searches response body for error messages indicating FPD.
+        Simple FPD test via cookie injection.
+        If cookie_name is provided -> test only that one (2 values: empty + invalid).
+        Otherwise, iterate self.cookie_names_list and test each name with these 2 values.
         """
-        # Define test templates
-        test_templates = {
-            "FPD detected when cookie value is empty": "",
-            "FPD detected when cookie value contains invalid characters": "aaa;bbb",
-        }
-
-        # Build test cases for each cookie name
-        test_cases = {}
-        for name in self.cookie_names_list:
-            for title, value in test_templates.items():
-                key = f"{title}_{name}"
-                test_cases[key] = {"cookie": {name: value}, "title": title}
 
         # Error patterns to detect FPD
         error_patterns = [
@@ -436,12 +422,13 @@ class CookieTester:
             r"<b>Notice</b>: .* on line.*",
         ]
 
-        fpd_findings = {}
+        #fpd_findings = {}
 
-        # Execute test cases
-        for case_name, data in test_cases.items():
-            cookies = data["cookie"]
-            title = data["title"]
+        # test values: empty + invalid (semicolon)
+        test_values = ["", "aaa;bbb"]
+
+        for value in test_values:
+            cookies = {cookie_name: value}
 
             try:
                 headers_with_cookie = dict(self.args.headers or {})
@@ -461,33 +448,20 @@ class CookieTester:
                 )
 
                 # Check against error patterns
-                if any(re.search(pattern, response.text, re.IGNORECASE) for pattern in error_patterns):
-                    fpd_findings.setdefault(title, []).append((cookies, dump))
+                for pattern in error_patterns:
+                    match = re.search(pattern, response.text, re.IGNORECASE)
+                    if match:
+                        path_regex = re.compile(r"in <b>(.*?)</b>\s+on line", re.IGNORECASE)
+                        matches = path_regex.findall(match.group(0))
+                        if matches:
+                            # path only
+                            ptprinthelper.ptprint(matches[0], bullet_type="ADDITIONS", condition=not self.use_json, indent=12, colortext=True )
+
+                        else:
+                            # full error
+                            ptprinthelper.ptprint(match.group(0), bullet_type="ADDITIONS", condition=not self.use_json, indent=8, colortext=True)
+
 
             except requests.RequestException:
                 # Skip if the server rejects the request
                 continue
-
-        # Print grouped findings (title only once)
-        for title, results in fpd_findings.items():
-            ptprinthelper.ptprint(" ", bullet_type="TEXT", condition=not self.use_json)
-            ptprinthelper.ptprint(
-                title,
-                bullet_type="TITLE",
-                condition=not self.use_json,
-                colortext=True,
-            )
-            for cookies, dump in results:
-                note = f"Vulnerable cookie: {cookies}"
-                self.ptjsonlib.add_vulnerability(
-                    vuln_code="PTV-WEB-FPD-COOKIE",
-                    note=note,
-                    vuln_request=dump["request"],
-                    vuln_response=dump["response"],
-                    node_key=self.node_key
-                )
-                ptprinthelper.ptprint(
-                    note,
-                    bullet_type="TEXT",
-                    condition=not self.use_json,
-                )
